export declare class Matrix {
    private _n;
    private _m;
    private _data;
    /**
     * Represents a matrix
     * @constructor
     * @param {number[][]} data an array of array forming the matrix. eg. [ [1, 2], [3, 4] ]
     * @throws when one row isn't the same size as the others
     */
    constructor(data: number[][]);
    /**
     * @returns {number} returns the number of rows in the matrix
     */
    readonly n: number;
    /**
     * @returns {number} returns the number of columns in the matrix
     */
    readonly m: number;
    /**
     * @returns {string} returns a string representation of the matrix
     */
    toString(): string;
    /**
     * Add another matrix to this one. Matrices must be the same size
     * @param {Matrix} secondMatrix the matrix to add to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    add(secondMatrix: Matrix): Matrix;
    /**
     * Mutiply another matrix to this one. If first matrix is of size n by m, second matrix must be of size m by n.
     * @param {Matrix} secondMatrix the matrix to multiply to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    multiply(secondMatrix: Matrix): Matrix;
    /**
     * Mutiply the matrix by a scalar
     * @param {number} scalar the scalar to multiply the matrix by
     * @returns {Matrix} returns the new matrix
     */
    multiplyByScalar(scalar: number): Matrix;
    /**
     * Transpose the matrix
     * @returns {Matrix} returns the transpose of the matrix
     */
    transpose(): Matrix;
    /**
     * Check if the matrix is a square matrix
     * @returns {boolean} returns true if the matrix is a square matrix
     */
    isSquare(): boolean;
    private areAllRowsSameLength;
    /**
     * Create the identity matrix of size m
     * @param {number} size the size of the identity matrix
     * @returns {Matrix} returns the new matrix
     */
    static identity(size: number): Matrix;
    /**
     * Static alternative to toString
     * @param {Matrix} matrix the matrix to stringify
     * @returns {string} returns a string representation of the matrix
     */
    static toString(matrix: Matrix): string;
    /**
     * Static alternative to add. Matrices must be the same size
     * @param {Matrix} matrix1 the first matrix
     * @param {Matrix} matrix2 the matrix to add to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    static add(matrix1: Matrix, matrix2: Matrix): Matrix;
    /**
     * Static alternative to multiply. If first matrix is of size n by m, second matrix must be of size m by n.
     * @param {Matrix} matrix1 the first matrix
     * @param {Matrix} matrix2 the matrix to multiply to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    static multiply(matrix1: Matrix, matrix2: Matrix): Matrix;
    /**
     * Static alternative to multiplyByScalar
     * @param {Matrix} matrix the matrix to multiply by the scalar
     * @param {number} scalar the scalar to multiply the matrix by
     * @returns {Matrix} returns the new matrix
     */
    static multiplyByScalar(matrix: Matrix, scalar: number): Matrix;
    /**
     * Static alternative to transpose
     * @param {Matrix} matrix the matrix to transpose
     * @returns {Matrix} returns the transpose of the matrix
     */
    static transpose(matrix: Matrix): Matrix;
    /**
     * Static alternative to isSquare
     * @param {Matrix} matrix the matrix to transpose
     * @returns {boolean} returns true if the matrix is a square matrix
     */
    static isSquare(matrix: Matrix): boolean;
}
