"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Matrix = /** @class */ (function () {
    /**
     * Represents a matrix
     * @constructor
     * @param {number[][]} data an array of array forming the matrix. eg. [ [1, 2], [3, 4] ]
     * @throws when one row isn't the same size as the others
     */
    function Matrix(data) {
        if (!this.areAllRowsSameLength(data))
            throw new Error("Rows have to be the same length");
        this._data = data;
        this._n = data.length;
        this._m = data[0].length;
    }
    Object.defineProperty(Matrix.prototype, "n", {
        /**
         * @returns {number} returns the number of rows in the matrix
         */
        get: function () {
            return this._n;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Matrix.prototype, "m", {
        /**
         * @returns {number} returns the number of columns in the matrix
         */
        get: function () {
            return this._m;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @returns {string} returns a string representation of the matrix
     */
    Matrix.prototype.toString = function () {
        var str = "";
        for (var i = 0; i < this._n; i++) {
            for (var j = 0; j < this._m; j++) {
                str += this._data[i][j];
                if (j !== this._m - 1)
                    str += " ";
            }
            if (i !== this._n - 1)
                str += "\n";
        }
        return str;
    };
    /**
     * Add another matrix to this one. Matrices must be the same size
     * @param {Matrix} secondMatrix the matrix to add to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    Matrix.prototype.add = function (secondMatrix) {
        if (this._n !== secondMatrix._n || this._m !== secondMatrix._m)
            throw new Error("Incompatible size for add(). Tried to add " + this._n + "x" + this._m + " and " + secondMatrix._n + "x" + secondMatrix._m + ".");
        var data = [];
        for (var i = 0; i < this._n; i++) {
            data[i] = [];
            for (var j = 0; j < this._m; j++) {
                data[i][j] = this._data[i][j] + secondMatrix._data[i][j];
            }
        }
        return new Matrix(data);
    };
    /**
     * Mutiply another matrix to this one. If first matrix is of size n by m, second matrix must be of size m by n.
     * @param {Matrix} secondMatrix the matrix to multiply to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    Matrix.prototype.multiply = function (secondMatrix) {
        if (this._m !== secondMatrix._n)
            throw new Error("Incompatible size for multiply(). Tried to multiply " + this._n + "x" + this._m + " and " + secondMatrix._n + "x" + secondMatrix._m + ".");
        var data = [];
        for (var i = 0; i < this._n; i++) {
            data[i] = [];
            for (var j = 0; j < secondMatrix._m; j++) {
                data[i][j] = 0;
                for (var k = 0; k < this._m; k++) {
                    data[i][j] += this._data[i][k] * secondMatrix._data[k][j];
                }
            }
        }
        return new Matrix(data);
    };
    /**
     * Mutiply the matrix by a scalar
     * @param {number} scalar the scalar to multiply the matrix by
     * @returns {Matrix} returns the new matrix
     */
    Matrix.prototype.multiplyByScalar = function (scalar) {
        var data = [];
        for (var i = 0; i < this._n; i++) {
            data[i] = [];
            for (var j = 0; j < this._m; j++) {
                data[i][j] = this._data[i][j] * scalar;
            }
        }
        return new Matrix(data);
    };
    /**
     * Transpose the matrix
     * @returns {Matrix} returns the transpose of the matrix
     */
    Matrix.prototype.transpose = function () {
        var data = [];
        for (var j = 0; j < this._m; j++) {
            data[j] = [];
            for (var i = 0; i < this._n; i++) {
                data[j][i] = this._data[i][j];
            }
        }
        return new Matrix(data);
    };
    /**
     * Check if the matrix is a square matrix
     * @returns {boolean} returns true if the matrix is a square matrix
     */
    Matrix.prototype.isSquare = function () {
        return this._n === this._m;
    };
    Matrix.prototype.areAllRowsSameLength = function (data) {
        var length0 = data[0].length;
        for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
            var row = data_1[_i];
            if (length0 !== row.length)
                return false;
        }
        return true;
    };
    /*--------------------------------------------------------------------------------
                STATIC METHODS
    --------------------------------------------------------------------------------*/
    /**
     * Create the identity matrix of size m
     * @param {number} size the size of the identity matrix
     * @returns {Matrix} returns the new matrix
     */
    Matrix.identity = function (size) {
        var data = [];
        for (var i = 0; i < size; i++) {
            data[i] = [];
            for (var j = 0; j < size; j++) {
                if (i === j)
                    data[i][j] = 1;
                else
                    data[i][j] = 0;
            }
        }
        return new Matrix(data);
    };
    /**
     * Static alternative to toString
     * @param {Matrix} matrix the matrix to stringify
     * @returns {string} returns a string representation of the matrix
     */
    Matrix.toString = function (matrix) {
        return matrix.toString();
    };
    /**
     * Static alternative to add. Matrices must be the same size
     * @param {Matrix} matrix1 the first matrix
     * @param {Matrix} matrix2 the matrix to add to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    Matrix.add = function (matrix1, matrix2) {
        return matrix1.add(matrix2);
    };
    /**
     * Static alternative to multiply. If first matrix is of size n by m, second matrix must be of size m by n.
     * @param {Matrix} matrix1 the first matrix
     * @param {Matrix} matrix2 the matrix to multiply to the first one
     * @returns {Matrix} returns the new matrix
     * @throws when the two matrices are of incompatible size
     */
    Matrix.multiply = function (matrix1, matrix2) {
        return matrix1.multiply(matrix2);
    };
    /**
     * Static alternative to multiplyByScalar
     * @param {Matrix} matrix the matrix to multiply by the scalar
     * @param {number} scalar the scalar to multiply the matrix by
     * @returns {Matrix} returns the new matrix
     */
    Matrix.multiplyByScalar = function (matrix, scalar) {
        return matrix.multiplyByScalar(scalar);
    };
    /**
     * Static alternative to transpose
     * @param {Matrix} matrix the matrix to transpose
     * @returns {Matrix} returns the transpose of the matrix
     */
    Matrix.transpose = function (matrix) {
        return matrix.transpose();
    };
    /**
     * Static alternative to isSquare
     * @param {Matrix} matrix the matrix to transpose
     * @returns {boolean} returns true if the matrix is a square matrix
     */
    Matrix.isSquare = function (matrix) {
        return matrix.isSquare();
    };
    return Matrix;
}());
exports.Matrix = Matrix;
