"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Vector = /** @class */ (function () {
    /**
     * Represents a Vector
     * @constructor
     * @param {number[]} data an array forming the vector. eg. [1, 2]
     */
    function Vector(data) {
        this._n = data.length;
        this._data = data;
    }
    Object.defineProperty(Vector.prototype, "dimension", {
        /**
         * @returns {number} returns the number dimensions of the vector
         */
        get: function () {
            return this._n;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @returns {string} returns a string representation of the vector
     */
    Vector.prototype.toString = function () {
        var str = "(";
        for (var i = 0; i < this._n; i++) {
            str += this._data[i];
            if (i !== this._n - 1)
                str += ", ";
        }
        return str + ")";
    };
    /**
     * Add another vector to this one. Vectors must be the same size
     * @param {Vector} vector the vector to add to the first one
     * @returns {Vector} returns the new vector
     * @throws when the two vectors are of incompatible size
     */
    Vector.prototype.add = function (vector) {
        if (this._n !== vector._n)
            throw new Error("Incompatible size for add(). Tried to add " + this._n + " and " + vector._n + ".");
        var data = [];
        for (var i = 0; i < this._n; i++) {
            data[i] = this._data[i] + vector._data[i];
        }
        return new Vector(data);
    };
    /**
     * Mutiply another vector to this one. Vectors must be the same size
     * @param {Vector} vector the vector to multiply to the first one
     * @returns {number} returns the product of the two vectors
     * @throws when the two vectors are of incompatible size
     */
    Vector.prototype.multiply = function (vector) {
        if (this._n !== vector._n)
            throw new Error("Incompatible size for multiply(). Tried to multiply " + this._n + " and " + vector._n + ".");
        var result = 0;
        for (var i = 0; i < this._n; i++) {
            result += this._data[i] * vector._data[i];
        }
        return result;
    };
    /**
     * Mutiply the vector by a scalar
     * @param {number} scalar the scalar to multiply the vector by
     * @returns {Vector} returns the new vector
     */
    Vector.prototype.multiplyByScalar = function (scalar) {
        var data = [];
        for (var i = 0; i < this._n; i++) {
            data[i] = this._data[i] * scalar;
            ;
        }
        return new Vector(data);
    };
    /*--------------------------------------------------------------------------------
                STATIC METHODS
    --------------------------------------------------------------------------------*/
    /**
     * Static alternative to toString
     * @returns {string} returns a string representation of the vector
     */
    Vector.toString = function (vector) {
        return vector.toString();
    };
    /**
     * Static alternative to add. Vectors must be the same size
     * @param {Vector} vector1 the first vector to add
     * @param {Vector} vector2 the second vector to add
     * @returns {Vector} returns the new vector
     * @throws when the two vectors are of incompatible size
     */
    Vector.add = function (vector1, vector2) {
        return vector1.add(vector2);
    };
    /**
     * Static alternative to multiply. Vectors must be the same size
     * @param {Vector} vector1 the first vector to multiply
     * @param {Vector} vector2 the second vector to multiply
     * @returns {number} returns the product of the two vectors
     * @throws when the two vectors are of incompatible size
     */
    Vector.multiply = function (vector1, vector2) {
        return vector1.multiply(vector2);
    };
    /**
     * Static alternative to multiplyByScalar
     * @param {Vector} vector the vector to multiply by the scalar
     * @param {number} scalar the scalar to multiply the vector by
     * @returns {Vector} returns the new vector
     */
    Vector.multiplyByScalar = function (vector, scalar) {
        return vector.multiplyByScalar(scalar);
    };
    return Vector;
}());
exports.Vector = Vector;
