"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Fraction = /** @class */ (function () {
    /**
     * Represents a fraction
     * @constructor
     * @param {number} numerator the numerator of the fraction
     * @param {number} denominator the denominator of the fraction
     * @throws when the numerator or the denominator is not an integer
     * @throws when the denominator is zero
     */
    function Fraction(numerator, denominator) {
        if (!this.isInteger(numerator) || !this.isInteger(denominator))
            throw new Error("Numerator and denominator should be integers");
        if (denominator === 0)
            throw new Error("Denominator can't be 0");
        this.numerator = numerator;
        this.denominator = denominator;
    }
    /**
     * @returns {string} returns a string representation of the fraction
     */
    Fraction.prototype.toString = function () {
        return this.numerator + "/" + this.denominator;
    };
    /**
     * @returns {number} returns the real value of the fraction
     */
    Fraction.prototype.eval = function () {
        return this.numerator / this.denominator;
    };
    /**
     * Reduce the fraction. For example, 6/4 will become 3/2
     * @returns {Fraction} returns reduced fraction
     */
    Fraction.prototype.reduce = function () {
        var gcd = this.getGreatestCommonDivisor(this.numerator, this.denominator);
        return new Fraction(this.numerator / gcd, this.denominator / gcd);
    };
    /**
     * Add another fraction to this one
     * @param {Fraction} fraction the fraction to add to the first one
     * @returns {Fraction} returns the new fraction
     */
    Fraction.prototype.add = function (fraction) {
        var newFirstNumerator = this.numerator * fraction.denominator;
        var newSecondNumerator = fraction.numerator * this.denominator;
        var commonDivisor = this.denominator * fraction.denominator;
        var result = new Fraction(newFirstNumerator + newSecondNumerator, commonDivisor);
        return result.reduce();
    };
    /**
     * Multiply another fraction to this one
     * @param {Fraction} fraction the fraction to multiply to the first one
     * @returns {Fraction} returns the new fraction
     */
    Fraction.prototype.multiply = function (fraction) {
        var result = new Fraction(this.numerator * fraction.numerator, this.denominator * fraction.denominator);
        return result.reduce();
    };
    Fraction.prototype.isInteger = function (number) {
        return number % 1 === 0;
    };
    Fraction.prototype.getGreatestCommonDivisor = function (x, y) {
        if (x === 0)
            return y;
        return this.getGreatestCommonDivisor(y % x, x);
    };
    /*--------------------------------------------------------------------------------
                 STATIC METHODS
    --------------------------------------------------------------------------------*/
    /**
     * Static alternative to toString
     * @param {Fraction} fraction the fraction to print
     * @returns {string} returns a string representation of the fraction
     */
    Fraction.toString = function (fraction) {
        return fraction.toString();
    };
    /**
     * Static alternative to eval
     * @param {Fraction} fraction the fraction to eval
     * @returns {number} returns the real value of the fraction
     */
    Fraction.eval = function (fraction) {
        return fraction.eval();
    };
    /**
     * Static alternative to reduce
     * @param {Fraction} fraction the fraction to reduce
     * @returns {Fraction} returns reduced fraction
     */
    Fraction.reduce = function (fraction) {
        return fraction.reduce();
    };
    /**
     * Static alternative to add
     * @param {Fraction} fraction1 the first fraction to add
     * @param {Fraction} fraction2 the fraction to add to the first one
     * @returns {Fraction} returns the new fraction
     */
    Fraction.add = function (fraction1, fraction2) {
        return fraction1.add(fraction2);
    };
    /**
     * Static alternative to multiply
     * @param {Fraction} fraction1 the first fraction to multiply
     * @param {Fraction} fraction2 the fraction to multiply to the first one
     * @returns {Fraction} returns the new fraction
     */
    Fraction.multiply = function (fraction1, fraction2) {
        return fraction1.multiply(fraction2);
    };
    return Fraction;
}());
exports.Fraction = Fraction;
